from qgis.core import QgsSettings, QgsProject, \
    QgsVectorLayer, QgsFeature, QgsGeometry, QgsCoordinateReferenceSystem, \
    QgsRasterLayer
import os.path

class PAGGFUtils(object):
    def __init__(self, paggf_layers):
        self.layers = paggf_layers  # instance de LoadedLayers
        self.plugin_dir = os.path.dirname(__file__)

    def get_municipality(self, municipality_id):
        return self.layers.get_feature(
            "ref", "municipalities", municipality_id)

    def get_province_by_munip_id(self, municipality_id):
        municipality = self.get_municipality(municipality_id)
        if municipality is not None:
            return self.layers.get_feature(
                "ref", "provinces", municipality['id_province'])

    def get_current_municipality_id(self):
        s = QgsSettings()
        v = s.value("PAGGF/current_municipality", None)
        if v == "":
            v = None
        return v

    def get_current_municipality(self):
        municipality_id = self.get_current_municipality_id()
        if municipality_id:
            return self.get_municipality(municipality_id)

    def get_current_province(self):
        municipality = self.get_current_municipality()
        if municipality:
            return self.get_province_by_munip_id(municipality['id'])

    def get_encoded_idc_parcels_for(self, municipality):
        return self.layers.get_features_by_cond(
            'gis', 'encoded_idc_parcels', #TOOD idc_parcels
            (lambda f: f['id_sub_hill_avenue'][:4] == municipality['id'])
        )

    def get_mask_layer(self):
        for lid, l in QgsProject.instance().mapLayers().items():
            if l.providerType() == "memory" and l.customProperty("is_mask", False):
                return l
        return None

    def clear_mask_layer(self):
        mask_layer = self.get_mask_layer()
        if mask_layer is not None:
            QgsProject.instance().removeMapLayers([mask_layer.id()])

    def update_mask_layer(self, mun_id=None):
        if not mun_id:
            mun_id = self.get_current_municipality_id()
            if mun_id is None:
                self.clear_mask_layer()
                return

        f = self.layers.get_feature('ref', 'municipalities', mun_id)
        if f is None:
            self.clear_mask_layer()
            return

        mask_layer = self.get_mask_layer()
        if mask_layer is None:
            l = QgsVectorLayer("MultiPolygon?crs=EPSG:32735", "Commune courante", "memory")
            l.setCustomProperty("is_mask", True)

            new_feature = QgsFeature()
            new_feature.setId(1)
            new_feature.setGeometry(QgsGeometry(f.geometry()))
            l.dataProvider().addFeatures([new_feature])

            l = QgsProject.instance().layerStore().addMapLayer(l)
            QgsProject.instance().layerTreeRoot().insertLayer(0, l)

            l.loadNamedStyle(os.path.join(os.path.dirname(__file__), "layer_style", "mask.qml"))
        else:
            mask_layer.dataProvider().changeGeometryValues({1: f.geometry()})

def update_background_layer():
    "(Re)load the background raster layer"
    s = QgsSettings()
    ortho = s.value("PAGGF/ortho", "")
    ok = False
    if ortho:
        bg_lyr = QgsRasterLayer(ortho, "Orthophoto")
        bg_lyr.setCrs(QgsCoordinateReferenceSystem("EPSG:32735"))
        ok = bg_lyr.isValid()
    if not ok:
        bg_lyr = QgsRasterLayer("crs=EPSG:3857&format&type=xyz&url=http://b.tile.openstreetmap.org/{z}/{x}/{y}.png&zmax=18&zmin=0", "Fond OSM", "wms")

    # remove all raster layers
    for l in QgsProject.instance().mapLayers().values():
        if isinstance(l, QgsRasterLayer):
            QgsProject.instance().removeMapLayer(l)
    # add to the end of the layer stack
    bg_lyr = QgsProject.instance().layerStore().addMapLayer(bg_lyr)
    QgsProject.instance().layerTreeRoot().addLayer(bg_lyr)

def get_background_layer():
    for l in QgsProject.instance().mapLayers().values():
        if isinstance(l, QgsRasterLayer):
            return l
